@@ -17,15 +17,10 @@ if ($_COOKIE["adminer_permanent"]) {
 
 function add_invalid_login() {
 	global $adminer;
-	$filename = get_temp_dir() . "/adminer.invalid";
-	$fp = @fopen($filename, "r+"); // @ - may not exist
-	if (!$fp) { // c+ is available since PHP 5.2.6
-		$fp = @fopen($filename, "w"); // @ - may not be writable
-		if (!$fp) {
-			return;
-		}
+	$fp = file_open_lock(get_temp_dir() . "/adminer.invalid");
+	if (!$fp) {
+		return;
 	}
-	flock($fp, LOCK_EX);
 	$invalids = unserialize(stream_get_contents($fp));
 	$time = time();
 	if ($invalids) {
@@ -40,22 +35,21 @@ function add_invalid_login() {
 		$invalid = array($time + 30*60, 0); // active for 30 minutes
 	}
 	$invalid[1]++;
-	$serialized = serialize($invalids);
-	rewind($fp);
-	fwrite($fp, $serialized);
-	ftruncate($fp, strlen($serialized));
-	flock($fp, LOCK_UN);
-	fclose($fp);
+	file_write_unlock($fp, serialize($invalids));
+}
+
+function check_invalid_login() {
+	global $adminer;
+	$invalids = unserialize(@file_get_contents(get_temp_dir() . "/adminer.invalid")); // @ - may not exist
+	$invalid = $invalids[$adminer->bruteForceKey()];
+	$next_attempt = ($invalid[1] > 29 ? $invalid[0] - time() : 0); // allow 30 invalid attempts
+	if ($next_attempt > 0) { //! do the same with permanent login
+		auth_error(lang('Too many unsuccessful logins, try again in %d minute(s).', ceil($next_attempt / 60)));
+	}
 }
 
 $auth = $_POST["auth"];
 if ($auth) {
-	$invalids = unserialize(@file_get_contents(get_temp_dir() . "/adminer.invalid")); // @ - may not exist
-	$invalid = $invalids[$adminer->bruteForceKey()];
-	$next_attempt = ($invalid[1] > 30 ? $invalid[0] - time() : 0); // allow 30 invalid attempts
-	if ($next_attempt > 0) { //! do the same with permanent login
-		auth_error(lang('Too many unsuccessful logins, try again in %d minute(s).', ceil($next_attempt / 60)));
-	}
 	session_regenerate_id(); // defense against session fixation
 	$vendor = $auth["driver"];
 	$server = $auth["server"];
@@ -89,7 +83,7 @@ if ($auth) {
 			set_session($key, null);
 		}
 		unset_permanent();
-		redirect(substr(preg_replace('~\b(username|db|ns)=[^&]*&~', '', ME), 0, -1), lang('Logout successful.'));
+		redirect(substr(preg_replace('~\b(username|db|ns)=[^&]*&~', '', ME), 0, -1), lang('Logout successful.') . ' ' . lang('Thanks for using Adminer, consider <a href="%s">donating</a>.', 'https://sourceforge.net/donate/index.php?group_id=264133'));
 	}
 	
 } elseif ($permanent && !$_SESSION["pwds"]) {
@@ -130,7 +124,7 @@ function auth_error($error) {
 			$password = get_password();
 			if ($password !== null) {
 				if ($password === false) {
-					$error .= '<br>' . lang('Master password expired. <a href="https://www.adminer.org/en/extension/" target="_blank">Implement</a> %s method to make it permanent.', '<code>permanentLogin()</code>');
+					$error .= '<br>' . lang('Master password expired. <a href="https://www.adminer.org/en/extension/"%s>Implement</a> %s method to make it permanent.', target_blank(), '<code>permanentLogin()</code>');
 				}
 				set_password(DRIVER, SERVER, $_GET["username"], null);
 			}
@@ -144,10 +138,12 @@ function auth_error($error) {
 	cookie("adminer_key", ($_COOKIE["adminer_key"] ? $_COOKIE["adminer_key"] : rand_string()), $params["lifetime"]);
 	page_header(lang('Login'), $error, null);
 	echo "<form action='' method='post'>\n";
-	$adminer->loginForm();
 	echo "<div>";
-	hidden_fields($_POST, array("auth")); // expired session
+	if (hidden_fields($_POST, array("auth"))) { // expired session
+		echo "<p class='message'>" . lang('The action will be performed after successful login with the same credentials.') . "\n";
+	}
 	echo "</div>\n";
+	$adminer->loginForm();
 	echo "</form>\n";
 	page_footer("auth");
 	exit;
@@ -161,11 +157,16 @@ if (isset($_GET["username"])) {
 		page_footer("auth");
 		exit;
 	}
+	list($host, $port) = explode(":", SERVER, 2);
+	if (is_numeric($port) && $port < 1024) {
+		auth_error(lang('Connecting to privileged ports is not allowed.'));
+	}
+	check_invalid_login();
 	$connection = connect();
+	$driver = new Min_Driver($connection);
 }
 
-$driver = new Min_Driver($connection);
-
+$login = null;
 if (!is_object($connection) || ($login = $adminer->login($_GET["username"], get_password())) !== true) {
 	auth_error((is_string($connection) ? h($connection) : (is_string($login) ? $login : lang('Invalid credentials.'))));
 }
