@@ -12,7 +12,7 @@ $translations = array(
 	'Logout successful.' => 'Αποσυνδεθήκατε με επιτυχία.',
 	'Invalid credentials.' => 'Εσφαλμένα Διαπιστευτήρια.',
 	'Too many unsuccessful logins, try again in %d minute(s).' => array('Επανηλημμένες ανεπιτυχείς προσπάθειες σύνδεσης, δοκιμάστε ξανά σε %s λεπτό.', 'Επανηλημμένες ανεπιτυχείς προσπάθειες σύνδεσης, δοκιμάστε ξανά σε %s λεπτά.'),
-	'Master password expired. <a href="https://www.adminer.org/en/extension/" target="_blank">Implement</a> %s method to make it permanent.' => 'Έλειξε ο Κύριος Κωδικός. <a href="https://www.adminer.org/en/extension/" target="_blank">Ενεργοποιήστε</a> τη μέθοδο %s για να τον κάνετε μόνιμο.',
+	'Master password expired. <a href="https://www.adminer.org/en/extension/"%s>Implement</a> %s method to make it permanent.' => 'Έλειξε ο Κύριος Κωδικός. <a href="https://www.adminer.org/en/extension/"%s>Ενεργοποιήστε</a> τη μέθοδο %s για να τον κάνετε μόνιμο.',
 	'Language' => 'Γλώσσα',
 	'Invalid CSRF token. Send the form again.' => 'Άκυρο κουπόνι CSRF. Στείλτε τη φόρμα ξανά.',
 	'If you did not send this request from Adminer then close this page.' => 'Αν δε στείλατε αυτό το αίτημα από το Adminer, τότε κλείστε αυτή τη σελίδα.',
@@ -149,7 +149,7 @@ $translations = array(
 	'Table' => 'Πίνακας',
 	'No tables.' => 'Χωρίς πίνακες.',
 	'Alter table' => 'Τροποποίηση πίνακα',
-	'Create table' => 'Δημιουργία  πίνακα',
+	'Create table' => 'Δημιουργία πίνακα',
 	'Table has been dropped.' => 'Ο πίνακας διαγράφηκε.',
 	'Tables have been dropped.' => 'Οι πινακες διαγράφηκαν.',
 	'Tables have been optimized.' => 'Οι πίνακες βελτιστοποιήθηκαν.',
@@ -246,7 +246,7 @@ $translations = array(
 	'last' => 'τελευταία',
 	'Load more data' => 'Φόρτωση κι άλλων δεδομένων',
 	'Loading' => 'Φορτώνει',
-	'whole result' => 'όλο το αποτέλεσμα',
+	'Whole result' => 'Όλο το αποτέλεσμα',
 	'%d byte(s)' => array('%d byte', '%d bytes'),
 	
 	'Import' => 'Εισαγωγή',
