@@ -8,7 +8,6 @@ $translations = array(
 	'Password' => 'パスワード',
 	'Select database' => 'データベースを選択してください',
 	'Invalid database.' => '不正なデータベース',
-	'Create new database' => '新規にデータベースを作成',
 	'Table has been dropped.' => 'テーブルを削除しました',
 	'Table has been altered.' => 'テーブルを変更しました',
 	'Table has been created.' => 'テーブルを作成しました',
@@ -185,7 +184,7 @@ $translations = array(
 	'Save and continue edit' => '保存して継続',
 	'original' => '元',
 	'%d item(s) have been affected.' => '%d を更新しました',
-	'whole result' => '全結果',
+	'Whole result' => '全結果',
 	'Tables have been dropped.' => 'テーブルを削除しました',
 	'Clone' => 'クローン',
 	'Maximum number of allowed fields exceeded. Please increase %s.' => '定義可能な最大フィールド数を越えました。%s を増やしてください。',
