@@ -185,7 +185,7 @@ $translations = array(
 	'Save and continue edit' => 'Сохранить и продолжить редактирование',
 	'original' => 'исходный',
 	'%d item(s) have been affected.' => array('Была изменена %d запись.', 'Были изменены %d записи.', 'Было изменено %d записей.'),
-	'whole result' => 'весь результат',
+	'Whole result' => 'Весь результат',
 	'Tables have been dropped.' => 'Таблицы были удалены.',
 	'Clone' => 'Клонировать',
 	'Partition by' => 'Разделить по',
@@ -269,9 +269,6 @@ $translations = array(
 	'Materialized view' => 'Материализованное представление',
 	'Vacuum' => 'Вакуум',
 	'Selected' => 'Выбранные',
-	'Replication' => 'Репликация',
-	'Master status' => 'состояние мастер-сервера',
-	'Slave status' => 'состояние ведомого сервера',
 	'File must be in UTF-8 encoding.' => 'Файл должен быть в кодировке UTF-8.',
 	'Modify' => 'Изменить',
 	'Loading' => 'Загрузка',
@@ -279,11 +276,11 @@ $translations = array(
 	'ATTACH queries are not supported.' => 'ATTACH-запросы не поддерживаются.',
 	'%d / ' => '%d / ',
 	'Limit rows' => 'Лимит строк',
-	'<a href="https://www.adminer.org/en/extension/" target="_blank">Implement</a> %s method to use SQLite.' => '<a href="https://www.adminer.org/en/extension/" target="_blank">Реализуйте</a> метод %s, чтобы использовать SQLite.',
+	'<a href="https://www.adminer.org/en/extension/"%s>Implement</a> %s method to use SQLite.' => '<a href="https://www.adminer.org/en/extension/"%s>Реализуйте</a> метод %s, чтобы использовать SQLite.',
 	'Default value' => 'Значение по умолчанию',
 	'Full table scan' => 'Анализ полной таблицы',
 	'Too many unsuccessful logins, try again in %d minute(s).' => array('Слишком много неудачных попыток входа. Попробуйте снова через %d минуту.', 'Слишком много неудачных попыток входа. Попробуйте снова через %d минуты.', 'Слишком много неудачных попыток входа. Попробуйте снова через %d минут.'),
-	'Master password expired. <a href="https://www.adminer.org/en/extension/" target="_blank">Implement</a> %s method to make it permanent.' => 'Мастер-пароль истёк. <a href="https://www.adminer.org/en/extension/" target="_blank">Реализуйте</a> метод %s, чтобы сделать его постоянным.',
+	'Master password expired. <a href="https://www.adminer.org/en/extension/"%s>Implement</a> %s method to make it permanent.' => 'Мастер-пароль истёк. <a href="https://www.adminer.org/en/extension/"%s>Реализуйте</a> метод %s, чтобы сделать его постоянным.',
 	'If you did not send this request from Adminer then close this page.' => 'Если вы не посылали этот запрос из Adminer, закройте эту страницу.',
 	'You can upload a big SQL file via FTP and import it from server.' => 'Вы можете закачать большой SQL-файл по FTP и затем импортировать его с сервера.',
 	'Size' => 'Размер',
