@@ -8,20 +8,23 @@ $translations = array(
 	'Permanent login' => 'Xx',
 	'Login' => 'Xx',
 	'Logout' => 'Xx',
-	'Logged as: %s' => 'Xx',
+	'Logged as: %s' => 'Xx: %s',
 	'Logout successful.' => 'Xx.',
+	'Thanks for using Adminer, consider <a href="%s">donating</a>.' => 'Xx <a href="%s">xx</a>.',
 	'Invalid credentials.' => 'Xx.',
-	'<a href="https://www.adminer.org/en/extension/" target="_blank">Implement</a> %s method to use SQLite.' => 'Xx.',
-	'Too many unsuccessful logins, try again in %d minute(s).' => array('Xx.', 'Xx.'),
-	'Master password expired. <a href="https://www.adminer.org/en/extension/" target="_blank">Implement</a> %s method to make it permanent.' => 'Xx.',
+	'<a href="https://www.adminer.org/en/extension/"%s>Implement</a> %s method to use SQLite.' => '<a href="https://www.adminer.org/en/extension/"%s>Xx</a> %s xx.',
+	'Too many unsuccessful logins, try again in %d minute(s).' => array('Xx %d.', 'Xx %d.'),
+	'Master password expired. <a href="https://www.adminer.org/en/extension/"%s>Implement</a> %s method to make it permanent.' => '<a href="https://www.adminer.org/en/extension/"%s>Xx</a> %s xx.',
 	'Language' => 'Xx',
 	'Invalid CSRF token. Send the form again.' => 'Xx.',
 	'If you did not send this request from Adminer then close this page.' => 'Xx.',
 	'No extension' => 'Xx',
-	'None of the supported PHP extensions (%s) are available.' => 'Xx.',
+	'None of the supported PHP extensions (%s) are available.' => 'Xx (%s).',
+	'Connecting to privileged ports is not allowed.' => 'Xx.',
 	'Session support must be enabled.' => 'Xx.',
 	'Session expired, please login again.' => 'Xx.',
-	'%s version: %s through PHP extension %s' => 'Xx',
+	'The action will be performed after successful login with the same credentials.' => 'Xx.',
+	'%s version: %s through PHP extension %s' => '%s xx: %s xx %s',
 	'Refresh' => 'Xx',
 	
 	// text direction - 'ltr' or 'rtl'
@@ -39,40 +42,37 @@ $translations = array(
 	'Revoke' => 'Xx',
 	
 	'Process list' => 'Xx',
-	'%d process(es) have been killed.' => array('Xx.', 'Xx.'),
+	'%d process(es) have been killed.' => array('%d xx.', '%d xx.'),
 	'Kill' => 'Xx',
 	
 	'Variables' => 'Xx',
 	'Status' => 'Xx',
 	
-	'Replication' => 'Xx',
-	'Master status' => 'Xx',
-	'Slave status' => 'Xx',
-	
 	'SQL command' => 'Xx',
-	'%d query(s) executed OK.' => array('Xx.', 'Xx.'),
-	'Query executed OK, %d row(s) affected.' => array('Xx.', 'Xx.'),
+	'%d query(s) executed OK.' => array('%d xx.', '%d xx.'),
+	'Query executed OK, %d row(s) affected.' => array('Xx, %d.', 'Xx, %d.'),
 	'No commands to execute.' => 'Xx.',
 	'Error in query' => 'Xx',
+	'Warnings' => 'Xx',
 	'ATTACH queries are not supported.' => 'Xx.',
 	'Execute' => 'Xx',
 	'Stop on error' => 'Xx',
 	'Show only errors' => 'Xx',
 	// sprintf() format for time of the command
-	'%.3f s' => 'xx',
+	'%.3f s' => '%.3f xx',
 	'History' => 'Xx',
 	'Clear' => 'Xx',
 	'Edit all' => 'Xx',
 	
 	'File upload' => 'Xx',
 	'From server' => 'Xx',
-	'Webserver file %s' => 'Xx',
+	'Webserver file %s' => 'Xx %s',
 	'Run file' => 'Xx',
 	'File does not exist.' => 'Xx.',
 	'File uploads are disabled.' => 'Xx.',
 	'Unable to upload a file.' => 'Xx.',
-	'Maximum allowed file size is %sB.' => 'Xx.',
-	'Too big POST data. Reduce the data or increase the %s configuration directive.' => 'Xx.',
+	'Maximum allowed file size is %sB.' => 'Xx %sB.',
+	'Too big POST data. Reduce the data or increase the %s configuration directive.' => 'Xx %s.',
 	'You can upload a big SQL file via FTP and import it from server.' => 'Xx.',
 	'You are offline.' => 'Xx.',
 	
@@ -86,6 +86,7 @@ $translations = array(
 	
 	'Database' => 'Xx',
 	'database' => 'xx',
+	'DB' => 'XX',
 	'Use' => 'Xx',
 	'Select database' => 'Xx',
 	'Invalid database.' => 'Xx.',
@@ -110,7 +111,7 @@ $translations = array(
 	'Index Length' => 'Xx',
 	'Data Free' => 'Xx',
 	'Rows' => 'Xx',
-	'%d in total' => 'xx',
+	'%d in total' => '%d xx',
 	'Analyze' => 'Xx',
 	'Optimize' => 'Xx',
 	'Vacuum' => 'Xx',
@@ -125,7 +126,7 @@ $translations = array(
 	'Tables have been copied.' => 'Xx.',
 	
 	'Routines' => 'Xx',
-	'Routine has been called, %d row(s) affected.' => array('Xx.', 'Xx.'),
+	'Routine has been called, %d row(s) affected.' => array('Xx, %d.', 'Xx, %d.'),
 	'Call' => 'Xx',
 	'Parameter name' => 'Xx',
 	'Create procedure' => 'Xx',
@@ -174,13 +175,14 @@ $translations = array(
 	'Default value' => 'Xx',
 	'Default values' => 'Xx',
 	'Drop' => 'Xx',
-	'Are you sure?' => 'Xx',
+	'Drop %s?' => 'Xx %s?',
+	'Are you sure?' => 'Xx?',
 	'Size' => 'Xx',
 	'Compute' => 'Xx',
 	'Move up' => 'Xx',
 	'Move down' => 'Xx',
 	'Remove' => 'Xx',
-	'Maximum number of allowed fields exceeded. Please increase %s.' => 'Xx.',
+	'Maximum number of allowed fields exceeded. Please increase %s.' => 'Xx %s.',
 	
 	'Partition by' => 'Xx',
 	'Partitions' => 'Xx',
@@ -246,17 +248,17 @@ $translations = array(
 	'Full table scan' => 'Xx',
 	'Unable to select the table' => 'Xx',
 	'No rows.' => 'Xx.',
-	'%d / ' => 'xx',
-	'%d row(s)' => array('xx', 'xx'),
+	'%d / ' => '%d / ',
+	'%d row(s)' => array('%d xx', '%d xx'),
 	'Page' => 'Xx',
 	'last' => 'xx',
 	'Load more data' => 'Xx',
 	'Loading' => 'Xx',
-	'whole result' => 'xx',
-	'%d byte(s)' => array('xx', 'xx'),
+	'Whole result' => 'Xx',
+	'%d byte(s)' => array('%d xx', '%d xx'),
 	
 	'Import' => 'Xx',
-	'%d row(s) have been imported.' => array('Xx.', 'Xx.'),
+	'%d row(s) have been imported.' => array('%d xx.', '%d xx.'),
 	'File must be in UTF-8 encoding.' => 'Xx.',
 	
 	// in-place editing in select
@@ -265,10 +267,10 @@ $translations = array(
 	'Use edit link to modify this value.' => 'Xx.',
 	
 	// %s can contain auto-increment value
-	'Item%s has been inserted.' => 'Xx.',
+	'Item%s has been inserted.' => 'Xx%s.',
 	'Item has been deleted.' => 'Xx.',
 	'Item has been updated.' => 'Xx.',
-	'%d item(s) have been affected.' => array('Xx.', 'Xx.'),
+	'%d item(s) have been affected.' => array('%d xx.', '%d xx.'),
 	'New item' => 'Xx',
 	'original' => 'xx',
 	// label for value '' in enum data type
@@ -289,7 +291,7 @@ $translations = array(
 	'Subject' => 'Xx',
 	'Attachments' => 'Xx',
 	'Send' => 'Xx',
-	'%d e-mail(s) have been sent.' => array('Xx.', 'Xx.'),
+	'%d e-mail(s) have been sent.' => array('%d xx.', '%d xx.'),
 	
 	// data type descriptions
 	'Numbers' => 'Xx',
@@ -314,7 +316,7 @@ $translations = array(
 	
 	// general SQLite error in create, drop or rename database
 	'File exists.' => 'Xx.',
-	'Please use one of the extensions %s.' => 'Xx.',
+	'Please use one of the extensions %s.' => 'Xx %s.',
 	
 	// PostgreSQL and MS SQL schema support
 	'Alter schema' => 'Xx',
