@@ -2,7 +2,7 @@
 if (support("kill") && $_POST && !$error) {
 	$killed = 0;
 	foreach ((array) $_POST["kill"] as $val) {
-		if (queries("KILL " . number($val))) {
+		if (kill_process($val)) {
 			$killed++;
 		}
 	}
@@ -13,8 +13,9 @@ page_header(lang('Process list'), $error);
 ?>
 
 <form action="" method="post">
-<table cellspacing="0" onclick="tableClick(event);" ondblclick="tableClick(event, true);" class="nowrap checkable">
+<table cellspacing="0" class="nowrap checkable">
 <?php
+echo script("mixin(qsl('table'), {onclick: tableClick, ondblclick: partialArg(tableClick, true)});");
 // HTML valid because there is always at least one process
 $i = -1;
 foreach (process_list() as $i => $row) {
@@ -30,8 +31,7 @@ foreach (process_list() as $i => $row) {
 		}
 		echo "</thead>\n";
 	}
-
-	echo "<tr" . odd() . ">" . (support("kill") ? "<td>" . checkbox("kill[]", $row["Id"], 0) : "");
+	echo "<tr" . odd() . ">" . (support("kill") ? "<td>" . checkbox("kill[]", $row[$jush == "sql" ? "Id" : "pid"], 0) : "");
 	foreach ($row as $key => $val) {
 		echo "<td>" . (
 			($jush == "sql" && $key == "Info" && preg_match("~Query|Killed~", $row["Command"]) && $val != "") ||
@@ -45,13 +45,13 @@ foreach (process_list() as $i => $row) {
 }
 ?>
 </table>
-<script type='text/javascript'>tableCheck();</script>
 <p>
 <?php
 if (support("kill")) {
-	echo ($i + 1) . "/" . lang('%d in total', $connection->result("SELECT @@max_connections"));
+	echo ($i + 1) . "/" . lang('%d in total', max_connections());
 	echo "<p><input type='submit' value='" . lang('Kill') . "'>\n";
 }
 ?>
 <input type="hidden" name="token" value="<?php echo $token; ?>">
 </form>
+<?php echo script("tableCheck();"); ?>
