@@ -4,11 +4,7 @@ $table_status = table_status1($TABLE);
 $indexes = indexes($TABLE);
 $fields = fields($TABLE);
 $foreign_keys = column_foreign_keys($TABLE);
-$oid = "";
-if ($table_status["Oid"]) {
-	$oid = ($jush == "sqlite" ? "rowid" : "oid");
-	$indexes[] = array("type" => "PRIMARY", "columns" => array($oid));
-}
+$oid = $table_status["Oid"];
 parse_str($_COOKIE["adminer_import"], $adminer_import);
 
 $rights = array(); // privilege => 0
@@ -30,10 +26,6 @@ $is_group = count($group) < count($select);
 $where = $adminer->selectSearchProcess($fields, $indexes);
 $order = $adminer->selectOrderProcess($fields, $indexes);
 $limit = $adminer->selectLimitProcess();
-$from = ($select ? implode(", ", $select) : "*" . ($oid ? ", $oid" : ""))
-	. convert_fields($columns, $fields, $select)
-	. "\nFROM " . table($TABLE);
-$group_by = ($group && $is_group ? "\nGROUP BY " . implode(", ", $group) : "") . ($order ? "\nORDER BY " . implode(", ", $order) : "");
 
 if ($_GET["val"] && is_ajax()) {
 	header("Content-Type: text/plain; charset=utf-8");
@@ -49,6 +41,24 @@ if ($_GET["val"] && is_ajax()) {
 	exit;
 }
 
+$primary = $unselected = null;
+foreach ($indexes as $index) {
+	if ($index["type"] == "PRIMARY") {
+		$primary = array_flip($index["columns"]);
+		$unselected = ($select ? $primary : array());
+		foreach ($unselected as $key => $val) {
+			if (in_array(idf_escape($key), $select)) {
+				unset($unselected[$key]);
+			}
+		}
+		break;
+	}
+}
+if ($oid && !$primary) {
+	$primary = $unselected = array($oid => 0);
+	$indexes[] = array("type" => "PRIMARY", "columns" => array($oid));
+}
+
 if ($_POST && !$error) {
 	$where_check = $where;
 	if (!$_POST["all"] && is_array($_POST["check"])) {
@@ -59,25 +69,15 @@ if ($_POST && !$error) {
 		$where_check[] = "((" . implode(") OR (", $checks) . "))";
 	}
 	$where_check = ($where_check ? "\nWHERE " . implode(" AND ", $where_check) : "");
-	$primary = $unselected = null;
-	foreach ($indexes as $index) {
-		if ($index["type"] == "PRIMARY") {
-			$primary = array_flip($index["columns"]);
-			$unselected = ($select ? $primary : array());
-			break;
-		}
-	}
-	foreach ((array) $unselected as $key => $val) {
-		if (in_array(idf_escape($key), $select)) {
-			unset($unselected[$key]);
-		}
-	}
-
 	if ($_POST["export"]) {
 		cookie("adminer_import", "output=" . urlencode($_POST["output"]) . "&format=" . urlencode($_POST["format"]));
 		dump_headers($TABLE);
 		$adminer->dumpTable($TABLE, "");
-		if (!is_array($_POST["check"]) || $unselected === array()) {
+		$from = ($select ? implode(", ", $select) : "*")
+			. convert_fields($columns, $fields, $select)
+			. "\nFROM " . table($TABLE);
+		$group_by = ($group && $is_group ? "\nGROUP BY " . implode(", ", $group) : "") . ($order ? "\nORDER BY " . implode(", ", $order) : "");
+		if (!is_array($_POST["check"]) || $primary) {
 			$query = "SELECT $from$where_check$group_by";
 		} else {
 			$union = array();
@@ -108,7 +108,7 @@ if ($_POST && !$error) {
 				if ($_POST["clone"]) {
 					$query = "INTO " . table($TABLE) . " (" . implode(", ", array_keys($set)) . ")\nSELECT " . implode(", ", $set) . "\nFROM " . table($TABLE);
 				}
-				if ($_POST["all"] || ($unselected === array() && is_array($_POST["check"])) || $is_group) {
+				if ($_POST["all"] || ($primary && is_array($_POST["check"])) || $is_group) {
 					$result = ($_POST["delete"]
 						? $driver->delete($TABLE, $where_check)
 						: ($_POST["clone"]
@@ -124,8 +124,8 @@ if ($_POST && !$error) {
 						$result = ($_POST["delete"]
 							? $driver->delete($TABLE, $where2, 1)
 							: ($_POST["clone"]
-								? queries("INSERT" . limit1($query, $where2))
-								: $driver->update($TABLE, $set, $where2)
+								? queries("INSERT" . limit1($TABLE, $query, $where2))
+								: $driver->update($TABLE, $set, $where2, 1)
 							)
 						);
 						if (!$result) {
@@ -165,7 +165,7 @@ if ($_POST && !$error) {
 						$TABLE,
 						$set,
 						" WHERE " . ($where ? implode(" AND ", $where) . " AND " : "") . where_check($unique_idf, $fields),
-						!($is_group || $unselected === array()),
+						!$is_group && !$primary,
 						" "
 					);
 					if (!$result) {
@@ -226,7 +226,7 @@ $set = null;
 if (isset($rights["insert"]) || !support("table")) {
 	$set = "";
 	foreach ((array) $_GET["where"] as $val) {
-		if (count($foreign_keys[$val["col"]]) == 1 && ($val["op"] == "="
+		if ($foreign_keys[$val["col"]] && count($foreign_keys[$val["col"]]) == 1 && ($val["op"] == "="
 			|| (!$val["op"] && !preg_match('~[_%]~', $val["val"])) // LIKE in Editor
 		)) {
 			$set .= "&set" . urlencode("[" . bracket_escape($val["col"]) . "]") . "=" . urlencode($val["val"]);
@@ -259,17 +259,29 @@ if (!$columns && support("table")) {
 	}
 
 	$select2 = $select;
+	$group2 = $group;
 	if (!$select2) {
 		$select2[] = "*";
-		if ($oid) {
-			$select2[] = $oid;
+		$convert_fields = convert_fields($columns, $fields, $select);
+		if ($convert_fields) {
+			$select2[] = substr($convert_fields, 2);
 		}
 	}
-	$convert_fields = convert_fields($columns, $fields, $select);
-	if ($convert_fields) {
-		$select2[] = substr($convert_fields, 2);
+	foreach ($select as $key => $val) {
+		$field = $fields[idf_unescape($val)];
+		if ($field && ($as = convert_field($field))) {
+			$select2[$key] = "$as AS $val";
+		}
 	}
-	$result = $driver->select($TABLE, $select2, $where, $group, $order, $limit, $page, true);
+	if (!$is_group && $unselected) {
+		foreach ($unselected as $key => $val) {
+			$select2[] = idf_escape($key);
+			if ($group2) {
+				$group2[] = idf_escape($key);
+			}
+		}
+	}
+	$result = $driver->select($TABLE, $select2, $where, $group2, $order, $limit, $page, true);
 
 	if (!$result) {
 		echo "<p class='error'>" . error() . "\n";
@@ -288,7 +300,7 @@ if (!$columns && support("table")) {
 		}
 
 		// use count($rows) without LIMIT, COUNT(*) without grouping, FOUND_ROWS otherwise (slowest)
-		if ($_GET["page"] != "last" && +$limit && $group && $is_group && $jush == "sql") {
+		if ($_GET["page"] != "last" && $limit != "" && $group && $is_group && $jush == "sql") {
 			$found_rows = $connection->result(" SELECT FOUND_ROWS()"); // space to allow mysql.trace_mode
 		}
 
@@ -297,14 +309,18 @@ if (!$columns && support("table")) {
 		} else {
 			$backward_keys = $adminer->backwardKeys($TABLE, $table_name);
 
-			echo "<table id='table' cellspacing='0' class='nowrap checkable' onclick='tableClick(event);' ondblclick='tableClick(event, true);' onkeydown='return editingKeydown(event);'>\n";
-			echo "<thead><tr>" . (!$group && $select ? "" : "<td><input type='checkbox' id='all-page' onclick='formCheck(this, /check/);' class='jsonly'> <a href='" . h($_GET["modify"] ? remove_from_uri("modify") : $_SERVER["REQUEST_URI"] . "&modify=1") . "'>" . lang('Modify') . "</a>");
+			echo "<table id='table' cellspacing='0' class='nowrap checkable'>";
+			echo script("mixin(qs('#table'), {onclick: tableClick, ondblclick: partialArg(tableClick, true), onkeydown: editingKeydown});");
+			echo "<thead><tr>" . (!$group && $select
+				? ""
+				: "<td><input type='checkbox' id='all-page' class='jsonly'>" . script("qs('#all-page').onclick = partial(formCheck, /check/);", "")
+					. " <a href='" . h($_GET["modify"] ? remove_from_uri("modify") : $_SERVER["REQUEST_URI"] . "&modify=1") . "'>" . lang('Modify') . "</a>");
 			$names = array();
 			$functions = array();
 			reset($select);
 			$rank = 1;
 			foreach ($rows[0] as $key => $val) {
-				if ($key != $oid) {
+				if (!isset($unselected[$key])) {
 					$val = $_GET["columns"][key($select)];
 					$field = $fields[$select ? ($val ? $val["col"] : current($select)) : $key];
 					$name = ($field ? $adminer->fieldName($field, $rank) : ($val["fun"] ? "*" : $key));
@@ -314,13 +330,14 @@ if (!$columns && support("table")) {
 						$column = idf_escape($key);
 						$href = remove_from_uri('(order|desc)[^=]*|page') . '&order%5B0%5D=' . urlencode($key);
 						$desc = "&desc%5B0%5D=1";
-						echo '<th onmouseover="columnMouse(this);" onmouseout="columnMouse(this, \' hidden\');">';
+						echo "<th>" . script("mixin(qsl('th'), {onmouseover: partial(columnMouse), onmouseout: partial(columnMouse, ' hidden')});", "");
 						echo '<a href="' . h($href . ($order[0] == $column || $order[0] == $key || (!$order && $is_group && $group[0] == $column) ? $desc : '')) . '">'; // $order[0] == $key - COUNT(*)
 						echo apply_sql_function($val["fun"], $name) . "</a>"; //! columns looking like functions
 						echo "<span class='column hidden'>";
 						echo "<a href='" . h($href . $desc) . "' title='" . lang('descending') . "' class='text'> ↓</a>";
 						if (!$val["fun"]) {
-							echo '<a href="#fieldset-search" onclick="selectSearch(\'' . h(js_escape($key)) . '\'); return false;" title="' . lang('Search') . '" class="text jsonly"> =</a>';
+							echo '<a href="#fieldset-search" title="' . lang('Search') . '" class="text jsonly"> =</a>';
+							echo script("qsl('a').onclick = partial(selectSearch, '" . js_escape($key) . "');");
 						}
 						echo "</span>";
 					}
@@ -359,18 +376,22 @@ if (!$columns && support("table")) {
 				}
 				$unique_idf = "";
 				foreach ($unique_array as $key => $val) {
-					if (($jush == "sql" || $jush == "pgsql") && strlen($val) > 64) {
+					if (($jush == "sql" || $jush == "pgsql") && preg_match('~char|text|enum|set~', $fields[$key]["type"]) && strlen($val) > 64) {
 						$key = (strpos($key, '(') ? $key : idf_escape($key)); //! columns looking like functions
-						$key = "MD5(" . ($jush == 'sql' && preg_match("~^utf8_~", $fields[$key]["collation"]) ? $key : "CONVERT($key USING " . charset($connection) . ")") . ")";
+						$key = "MD5(" . ($jush != 'sql' || preg_match("~^utf8~", $fields[$key]["collation"]) ? $key : "CONVERT($key USING " . charset($connection) . ")") . ")";
 						$val = md5($val);
 					}
 					$unique_idf .= "&" . ($val !== null ? urlencode("where[" . bracket_escape($key) . "]") . "=" . urlencode($val) : "null%5B%5D=" . urlencode($key));
 				}
-				echo "<tr" . odd() . ">" . (!$group && $select ? "" : "<td>" . checkbox("check[]", substr($unique_idf, 1), in_array(substr($unique_idf, 1), (array) $_POST["check"]), "", "this.form['all'].checked = false; formUncheck('all-page');") . ($is_group || information_schema(DB) ? "" : " <a href='" . h(ME . "edit=" . urlencode($TABLE) . $unique_idf) . "'>" . lang('edit') . "</a>"));
+				echo "<tr" . odd() . ">" . (!$group && $select ? "" : "<td>"
+					. checkbox("check[]", substr($unique_idf, 1), in_array(substr($unique_idf, 1), (array) $_POST["check"]))
+					. ($is_group || information_schema(DB) ? "" : " <a href='" . h(ME . "edit=" . urlencode($TABLE) . $unique_idf) . "' class='edit'>" . lang('edit') . "</a>")
+				);
 
 				foreach ($row as $key => $val) {
 					if (isset($names[$key])) {
 						$field = $fields[$key];
+						$val = $driver->value($val, $field);
 						if ($val != "" && (!isset($email_fields[$key]) || $email_fields[$key] != "")) {
 							$email_fields[$key] = (is_mail($val) ? $names[$key] : ""); //! filled e-mails can be contained on other pages
 						}
@@ -419,7 +440,10 @@ if (!$columns && support("table")) {
 							echo "<td>" . ($text ? "<textarea name='$id' cols='30' rows='" . (substr_count($row[$key], "\n") + 1) . "'>$h_value</textarea>" : "<input name='$id' value='$h_value' size='$lengths[$key]'>");
 						} else {
 							$long = strpos($val, "<i>...</i>");
-							echo "<td id='$id' onclick=\"selectClick(this, event, " . ($long ? 2 : ($text ? 1 : 0)) . ($editable ? "" : ", '" . h(lang('Use edit link to modify this value.')) . "'") . ");\">$val";
+							echo "<td id='$id' data-text='" . ($long ? 2 : ($text ? 1 : 0)) . "'"
+								. ($editable ? "" : " data-warning='" . h(lang('Use edit link to modify this value.')) . "'")
+								. ">$val</td>"
+							;
 						}
 					}
 				}
@@ -437,103 +461,122 @@ if (!$columns && support("table")) {
 			echo "</table>\n";
 		}
 
-		if (($rows || $page) && !is_ajax()) {
-			$exact_count = true;
-			if ($_GET["page"] != "last") {
-				if (!+$limit) {
-					$found_rows = count($rows);
-				} elseif ($jush != "sql" || !$is_group) {
-					$found_rows = ($is_group ? false : found_rows($table_status, $where));
-					if ($found_rows < max(1e4, 2 * ($page + 1) * $limit)) {
-						// slow with big tables
-						$found_rows = reset(slow_query(count_rows($TABLE, $where, $is_group, $group)));
-					} else {
-						$exact_count = false;
+		if (!is_ajax()) {
+			if ($rows || $page) {
+				$exact_count = true;
+				if ($_GET["page"] != "last") {
+					if ($limit == "" || (count($rows) < $limit && ($rows || !$page))) {
+						$found_rows = ($page ? $page * $limit : 0) + count($rows);
+					} elseif ($jush != "sql" || !$is_group) {
+						$found_rows = ($is_group ? false : found_rows($table_status, $where));
+						if ($found_rows < max(1e4, 2 * ($page + 1) * $limit)) {
+							// slow with big tables
+							$found_rows = reset(slow_query(count_rows($TABLE, $where, $is_group, $group)));
+						} else {
+							$exact_count = false;
+						}
 					}
 				}
-			}
 
-			if (+$limit && ($found_rows === false || $found_rows > $limit || $page)) {
-				echo "<p class='pages'>";
-				// display first, previous 4, next 4 and last page
-				$max_page = ($found_rows === false
-					? $page + (count($rows) >= $limit ? 2 : 1)
-					: floor(($found_rows - 1) / $limit)
-				);
-				if ($jush != "simpledb") {
-					echo '<a href="' . h(remove_from_uri("page")) . "\" onclick=\"pageClick(this.href, +prompt('" . lang('Page') . "', '" . ($page + 1) . "'), event); return false;\">" . lang('Page') . "</a>:";
-					echo pagination(0, $page) . ($page > 5 ? " ..." : "");
-					for ($i = max(1, $page - 4); $i < min($max_page, $page + 5); $i++) {
-						echo pagination($i, $page);
-					}
-					if ($max_page > 0) {
-						echo ($page + 5 < $max_page ? " ..." : "");
-						echo ($exact_count && $found_rows !== false
-							? pagination($max_page, $page)
-							: " <a href='" . h(remove_from_uri("page") . "&page=last") . "' title='~$max_page'>" . lang('last') . "</a>"
-						);
-					}
+				$pagination = ($limit != "" && ($found_rows === false || $found_rows > $limit || $page));
+				if ($pagination) {
 					echo (($found_rows === false ? count($rows) + 1 : $found_rows - $page * $limit) > $limit
-						? ' <a href="' . h(remove_from_uri("page") . "&page=" . ($page + 1)) . '" onclick="return !selectLoadMore(this, ' . (+$limit) . ', \'' . lang('Loading') . '...\');" class="loadmore">' . lang('Load more data') . '</a>'
+						? '<p><a href="' . h(remove_from_uri("page") . "&page=" . ($page + 1)) . '" class="loadmore">' . lang('Load more data') . '</a>'
+							. script("qsl('a').onclick = partial(selectLoadMore, " . (+$limit) . ", '" . lang('Loading') . "...');", "")
 						: ''
 					);
-				} else {
-					echo lang('Page') . ":";
-					echo pagination(0, $page) . ($page > 1 ? " ..." : "");
-					echo ($page ? pagination($page, $page) : "");
-					echo ($max_page > $page ? pagination($page + 1, $page) . ($max_page > $page + 1 ? " ..." : "") : "");
+					echo "\n";
 				}
 			}
+			
+			echo "<div class='footer'><div>\n";
+			if ($rows || $page) {
+				if ($pagination) {
+					// display first, previous 4, next 4 and last page
+					$max_page = ($found_rows === false
+						? $page + (count($rows) >= $limit ? 2 : 1)
+						: floor(($found_rows - 1) / $limit)
+					);
+					echo "<fieldset>";
+					if ($jush != "simpledb") {
+						echo "<legend><a href='" . h(remove_from_uri("page")) . "'>" . lang('Page') . "</a></legend>";
+						echo script("qsl('a').onclick = function () { pageClick(this.href, +prompt('" . lang('Page') . "', '" . ($page + 1) . "')); return false; };");
+						echo pagination(0, $page) . ($page > 5 ? " ..." : "");
+						for ($i = max(1, $page - 4); $i < min($max_page, $page + 5); $i++) {
+							echo pagination($i, $page);
+						}
+						if ($max_page > 0) {
+							echo ($page + 5 < $max_page ? " ..." : "");
+							echo ($exact_count && $found_rows !== false
+								? pagination($max_page, $page)
+								: " <a href='" . h(remove_from_uri("page") . "&page=last") . "' title='~$max_page'>" . lang('last') . "</a>"
+							);
+						}
+					} else {
+						echo "<legend>" . lang('Page') . "</legend>";
+						echo pagination(0, $page) . ($page > 1 ? " ..." : "");
+						echo ($page ? pagination($page, $page) : "");
+						echo ($max_page > $page ? pagination($page + 1, $page) . ($max_page > $page + 1 ? " ..." : "") : "");
+					}
+					echo "</fieldset>\n";
+				}
+				
+				echo "<fieldset>";
+				echo "<legend>" . lang('Whole result') . "</legend>";
+				$display_rows = ($exact_count ? "" : "~ ") . $found_rows;
+				echo checkbox("all", 1, 0, ($found_rows !== false ? ($exact_count ? "" : "~ ") . lang('%d row(s)', $found_rows) : ""), "var checked = formChecked(this, /check/); selectCount('selected', this.checked ? '$display_rows' : checked); selectCount('selected2', this.checked || !checked ? '$display_rows' : checked);") . "\n";
+				echo "</fieldset>\n";
 
-			echo "<p class='count'>\n";
-			echo ($found_rows !== false ? "(" . ($exact_count ? "" : "~ ") . lang('%d row(s)', $found_rows) . ") " : "");
-			$display_rows = ($exact_count ? "" : "~ ") . $found_rows;
-			echo checkbox("all", 1, 0, lang('whole result'), "var checked = formChecked(this, /check/); selectCount('selected', this.checked ? '$display_rows' : checked); selectCount('selected2', this.checked || !checked ? '$display_rows' : checked);") . "\n";
-
-			if ($adminer->selectCommandPrint()) {
-				?>
+				if ($adminer->selectCommandPrint()) {
+					?>
 <fieldset<?php echo ($_GET["modify"] ? '' : ' class="jsonly"'); ?>><legend><?php echo lang('Modify'); ?></legend><div>
 <input type="submit" value="<?php echo lang('Save'); ?>"<?php echo ($_GET["modify"] ? '' : ' title="' . lang('Ctrl+click on a value to modify it.') . '"'); ?>>
 </div></fieldset>
 <fieldset><legend><?php echo lang('Selected'); ?> <span id="selected"></span></legend><div>
 <input type="submit" name="edit" value="<?php echo lang('Edit'); ?>">
 <input type="submit" name="clone" value="<?php echo lang('Clone'); ?>">
-<input type="submit" name="delete" value="<?php echo lang('Delete'); ?>"<?php echo confirm(); ?>>
+<input type="submit" name="delete" value="<?php echo lang('Delete'); ?>"><?php echo confirm(); ?>
 </div></fieldset>
 <?php
-			}
-
-			$format = $adminer->dumpFormat();
-			foreach ((array) $_GET["columns"] as $column) {
-				if ($column["fun"]) {
-					unset($format['sql']);
-					break;
 				}
-			}
-			if ($format) {
-				print_fieldset("export", lang('Export') . " <span id='selected2'></span>");
-				$output = $adminer->dumpOutput();
-				echo ($output ? html_select("output", $output, $adminer_import["output"]) . " " : "");
-				echo html_select("format", $format, $adminer_import["format"]);
-				echo " <input type='submit' name='export' value='" . lang('Export') . "'>\n";
-				echo "</div></fieldset>\n";
+
+				$format = $adminer->dumpFormat();
+				foreach ((array) $_GET["columns"] as $column) {
+					if ($column["fun"]) {
+						unset($format['sql']);
+						break;
+					}
+				}
+				if ($format) {
+					print_fieldset("export", lang('Export') . " <span id='selected2'></span>");
+					$output = $adminer->dumpOutput();
+					echo ($output ? html_select("output", $output, $adminer_import["output"]) . " " : "");
+					echo html_select("format", $format, $adminer_import["format"]);
+					echo " <input type='submit' name='export' value='" . lang('Export') . "'>\n";
+					echo "</div></fieldset>\n";
+				}
+
+				$adminer->selectEmailPrint(array_filter($email_fields, 'strlen'), $columns);
 			}
 
-			echo (!$group && $select ? "" : "<script type='text/javascript'>tableCheck();</script>\n");
+			echo "</div></div>\n";
+
+			if ($adminer->selectImportPrint()) {
+				echo "<div>";
+				echo "<a href='#import'>" . lang('Import') . "</a>";
+				echo script("qsl('a').onclick = partial(toggle, 'import');", "");
+				echo "<span id='import' class='hidden'>: ";
+				echo "<input type='file' name='csv_file'> ";
+				echo html_select("separator", array("csv" => "CSV,", "csv;" => "CSV;", "tsv" => "TSV"), $adminer_import["format"], 1); // 1 - select
+				echo " <input type='submit' name='import' value='" . lang('Import') . "'>";
+				echo "</span>";
+				echo "</div>";
+			}
+
+			echo "<input type='hidden' name='token' value='$token'>\n";
+			echo "</form>\n";
+			echo (!$group && $select ? "" : script("tableCheck();"));
 		}
-
-		if ($adminer->selectImportPrint()) {
-			print_fieldset("import", lang('Import'), !$rows);
-			echo "<input type='file' name='csv_file'> ";
-			echo html_select("separator", array("csv" => "CSV,", "csv;" => "CSV;", "tsv" => "TSV"), $adminer_import["format"], 1); // 1 - select
-			echo " <input type='submit' name='import' value='" . lang('Import') . "'>";
-			echo "</div></fieldset>\n";
-		}
-
-		$adminer->selectEmailPrint(array_filter($email_fields, 'strlen'), $columns);
-
-		echo "<p><input type='hidden' name='token' value='$token'></p>\n";
-		echo "</form>\n";
 	}
 }
 
