@@ -3,8 +3,8 @@
 /** Exports one database (e.g. development) so that it can be synced with other database (e.g. production)
 * @link https://www.adminer.org/plugins/#use
 * @author Jakub Vrana, https://www.vrana.cz/
-* @license http://www.apache.org/licenses/LICENSE-2.0 Apache License, Version 2.0
-* @license http://www.gnu.org/licenses/gpl-2.0.html GNU General Public License, version 2 (one or other)
+* @license https://www.apache.org/licenses/LICENSE-2.0 Apache License, Version 2.0
+* @license https://www.gnu.org/licenses/gpl-2.0.html GNU General Public License, version 2 (one or other)
 */
 class AdminerDumpAlter {
 	
@@ -69,7 +69,7 @@ SELECT @adminer_alter;
 	
 	function dumpTable($table, $style, $is_view = false) {
 		if ($_POST["format"] == "sql_alter") {
-			$create = create_sql($table, $_POST["auto_increment"]);
+			$create = create_sql($table, $_POST["auto_increment"], $style);
 			if ($is_view) {
 				echo substr_replace($create, " OR REPLACE", 6, 0) . ";\n\n";
 			} else {
