@@ -3,8 +3,8 @@
 /** Prefill field containing "_slug" with slugified value of a previous field (JavaScript)
 * @link https://www.adminer.org/plugins/#use
 * @author Jakub Vrana, https://www.vrana.cz/
-* @license http://www.apache.org/licenses/LICENSE-2.0 Apache License, Version 2.0
-* @license http://www.gnu.org/licenses/gpl-2.0.html GNU General Public License, version 2 (one or other)
+* @license https://www.apache.org/licenses/LICENSE-2.0 Apache License, Version 2.0
+* @license https://www.gnu.org/licenses/gpl-2.0.html GNU General Public License, version 2 (one or other)
 */
 class AdminerSlugify {
 	/** @access protected */
@@ -34,7 +34,16 @@ class AdminerSlugify {
 			}
 			$slug = $slugify[$field["field"]];
 			if ($slug !== null) {
-				return "<input value='" . h($value) . "' maxlength='$field[length]' size='40'$attrs onchange=\"var find = '$this->from'; var repl = '$this->to'; this.form['fields[$slug]'].value = this.value.toLowerCase().replace(new RegExp('[' + find + ']', 'g'), function (str) { return repl[find.indexOf(str)]; }).replace(/[^a-z0-9_]+/g, '-').replace(/^-|-\$/g, '').substr(0, $field[length]);\">";
+				return "<input value='" . h($value) . "' maxlength='$field[length]' size='40'$attrs>"
+					. script("qsl('input').onchange = function () {
+	var find = '$this->from';
+	var repl = '$this->to';
+	this.form['fields[$slug]'].value = this.value.toLowerCase()
+		.replace(new RegExp('[' + find + ']', 'g'), function (str) { return repl[find.indexOf(str)]; })
+		.replace(/[^a-z0-9_]+/g, '-')
+		.replace(/^-|-\$/g, '')
+		.substr(0, $field[length]);
+};");
 			}
 		}
 	}
